<?php

namespace App\Controller;

use App\Entity\Comment;
use App\Repository\CommentRepository;
use App\Repository\PageRepository;
use App\Repository\RedirectRepository;
use App\Service\CaptchaService;
use App\Service\PlaceholderService;
use App\Service\PushoverService;
use App\Service\S3Service;
use Symfony\Bundle\FrameworkBundle\Controller\AbstractController;
use Symfony\Component\HttpFoundation\RedirectResponse;
use Symfony\Component\HttpFoundation\Request;
use Symfony\Component\HttpFoundation\Response;
use Symfony\Component\Routing\Annotation\Route;

class PageController extends AbstractController
{
    #[Route('/{all}.html', requirements: ['all' => '.*'], methods: ['GET'], priority: -1)]
    public function redirectHtmlExtension(Request $request): Response
    {
        $path = $request->getPathInfo();
        if (str_ends_with($path, '.html')) {
            $path = substr($path, 0, strlen($path) - 5);
        }
        return new RedirectResponse($path);
    }

    #[Route('/{all}', requirements: ['all' => '.*'], methods: ['GET'], priority: -1)]
    public function renderPage(
        Request $request,
        PageRepository $pageRepository,
        CaptchaService $captchaService,
        PlaceholderService $placeholderService,
        RedirectRepository $redirectRepository,
        S3Service $s3Service
    ): Response {

        $page = $pageRepository->getPageFromURL($request->getPathInfo());
        if ($page) {
            $page = $placeholderService->replacePlaceholderInPage($page);
            return $this->render('template/' . $page->getTemplate() . '.html.twig', [
            'page' => $page,
            'captcha' => $captchaService->getInlineCaptcha(),
            'name' => '',
            'email' => '',
            'comment' => ''
            ]);
        }

        $redirect = $redirectRepository->getRedirectFromURL($request->getPathInfo());
        if ($redirect) {
            $url = $redirect->getRedirectURL();
            if ($redirect->isIsS3File()) {
                $url = $s3Service->generateSignedUrl('download/' . $url);
            }
            return new RedirectResponse($url);
        }


        return $this->render('pages/error404.html.twig')->setStatusCode(404);
    }

    #[Route('/{all}', requirements: ['all' => '.*'], methods: ['POST'], priority: -1)]
    public function newComment(
        Request $request,
        PageRepository $pageRepository,
        CommentRepository $commentRepository,
        CaptchaService $captchaService,
        PushoverService $pushoverService
    ): Response {
        $page = $pageRepository->getPageFromURL($request->getPathInfo());
        if (!$page) {
            return $this->render('pages/error404.html.twig')->setStatusCode(404);
        }

        if (strlen((string)$request->request->get('name')) > 255) {
            return $this->render('template/' . $page->getTemplate() . '.html.twig', [
                'page' => $page,
                'message' => 'Der Name darf maximal 255 Zeichen lang sein.',
                'messageColor' => 'red',
                'captcha' => $captchaService->getInlineCaptcha(),
                'name' => $request->request->get('name'),
                'email' => $request->request->get('email'),
                'comment' => $request->request->get('comment')
            ]);
        }

        if (strlen((string)$request->request->get('email')) > 255) {
            return $this->render('template/' . $page->getTemplate() . '.html.twig', [
                'page' => $page,
                'message' => 'Die E-Mail Adresse darf maximal 255 Zeichen lang sein.',
                'messageColor' => 'red',
                'captcha' => $captchaService->getInlineCaptcha(),
                'name' => $request->request->get('name'),
                'email' => $request->request->get('email'),
                'comment' => $request->request->get('comment')
            ]);
        }

        if (!$captchaService->validate((string)$request->request->get('captcha'))) {
            return $this->render('template/' . $page->getTemplate() . '.html.twig', [
                'page' => $page,
                'message' => 'Fehler beim verifizieren des Captchas.',
                'messageColor' => 'red',
                'captcha' => $captchaService->getInlineCaptcha(),
                'name' => $request->request->get('name'),
                'email' => $request->request->get('email'),
                'comment' => $request->request->get('comment')
            ]);
        }

        if (!$this->isCsrfTokenValid((string)$page->getId(), (string)$request->request->get('csrf'))) {
            return $this->render('template/' . $page->getTemplate() . '.html.twig', [
                'page' => $page,
                'message' => 'Fehler beim verifizieren des CSRF Tokens, bitte versuchen sie es erneut.',
                'messageColor' => 'red',
                'captcha' => $captchaService->getInlineCaptcha(),
                'name' => $request->request->get('name'),
                'email' => $request->request->get('email'),
                'comment' => $request->request->get('comment')
            ]);
        }

        $comment = new Comment();
        $comment->setName((string)$request->request->get('name'));
        $comment->setEmail((string)$request->request->get('email'));
        $comment->setComment((string)$request->request->get('comment'));
        $comment->setPage($page);
        $commentRepository->save($comment, true);

        $pushoverService->sendNotification(
            'Neuer Kommentar',
            'Es wurde ein neuer Kommentar auf der Seite ' . $page->getTitle() . ' veröffentlicht.',
            $request->getUri()
        );

        return $this->render('template/' . $page->getTemplate() . '.html.twig', [
            'page' => $page,
            'message' => 'Das Kommentar wurde erfolgreich veröffentlicht.',
            'messageColor' => 'green',
            'captcha' => $captchaService->getInlineCaptcha(),
            'name' => '',
            'email' => '',
            'comment' => ''
        ]);
    }
}
